--- ========================================================= ---
---  Setup Revision, Version, Date and add to AceLibrary
--- ========================================================= ---
local VERSION = tonumber(("$Revision: 44247 $"):match("%d+"))

local Sphere = Sphere
Sphere_CoverGauge = Sphere:NewModule("CoverGauge", "AceEvent-2.0")
local self = Sphere_CoverGauge
if Sphere.revision < VERSION then
	Sphere.version = "r" .. VERSION
	Sphere.revision = VERSION
	Sphere.date = ("$Date: 2007-07-19 15:21:49 -0400 (Thu, 19 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
Sphere.desc = "Adds a Cover Gauge to the Sphere"


--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local DogTag = AceLibrary("DogTag-1.0")

local L = Sphere:L("Sphere_CoverGauge")

local start = nil

local eventType = {
	health = "[PercentHP:Div(100)]",
	power = "[PercentMP:Div(100)]",
	threat = "[PercentThreat:Div(100)]",
	experience = "[PercentXP:Div(100)]",
	reputation = "[PercentRep:Div(100)]",
	none = "",
	custom = "",
}

--- ========================================================= ---
---  On Intitalize / Setup Database
--- ========================================================= ---
function Sphere_CoverGauge:OnInitialize()
	self.db = Sphere:AcquireDBNamespace("CoverGauge")
	Sphere:RegisterDefaults("CoverGauge", "char", {
		gaugeUnit = "player",
		gaugeType = "experience",
		gaugeCustom = "[PercentXP]",
		animate = true,
		useCustom = false,
		portrait = true,
	})

	self:CreateFrames()
	self:SetGaugePortrait()
end

--- ========================================================= ---
---  Setup Validates for Options
--- ========================================================= ---
Sphere_CoverGauge_eventTypeValidate = {
	health = L["Health"],
	power = L["Power"],
	threat = L["Threat"],
	experience = L["Experience"],
	reputation = L["Reputation"],
	custom = L["Custom"],
	none = L["None"],
}

local eventTypeValidateDesc = {}

local eventUnitValidate = {
	player = L["Player"],
	target = L["Target"],
	focus = L["Focus"],
	pet = L["Pet"],
}

local eventUnitValidateDesc = {}

--- ========================================================= ---
---  Setup Options
--- ========================================================= ---
Sphere_CoverGauge:RegisterSphereOption({
	name = L["Cover Gauge"],
	desc = L["Change the Cover Gauge options."],
	type = 'group',
	args = {
		event = {
			name = L["Event Type"],
			desc = L["Choose which Event Type the Gauge monitors and shows."],
			type = 'text',
			validate = Sphere_CoverGauge_eventTypeValidate,
			validateDesc = eventTypeValidateDesc,
			get = function()
				return self.db.char.gaugeType
			end,
			set = "SetGaugeType",
		},
		unit = {
			name = L["Event Unit"],
			desc = L["Choose which Unit the Event is monitoring and showing."],
			type = 'text',
			validate = eventUnitValidate,
			validateDesc = eventUnitValidateDesc,
			get = function()
				return self.db.char.gaugeUnit
			end,
			set = "SetGaugeUnit",
		},
		custom = {
			name = L["Custom Event"],
			desc = L["Create a Custom Event for the Gauge to monitor and show."],
			type = "text",
			get = function()
				return self.db.char.gaugeCustom
			end,
			set = "SetCustomGauge",
			usage = L["DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0"],
		},
		animate = {
			name = L["Animate Gauge"],
			desc = L["Animate the Gauge (Glide)."],
			type = 'toggle',
			get = function()
				return self.db.char.animate
			end,
			set = function() 
				self.db.char.animate = not(self.db.char.animate);
			end,
		},
		portrait = {
			name = L["Show Portrait"],
			desc = L["Show your Portrait."],
			type = 'toggle',
			get = function()
				return self.db.char.portrait
			end,
			set = function() 
				self.db.char.portrait = not(self.db.char.portrait);
				self:SetGaugePortrait();
			end,
		},
	}
	
})

--- ========================================================= ---
---  On Enable
--- ========================================================= ---
function Sphere_CoverGauge:OnEnable()
	self:RegisterEvent('UNIT_PORTRAIT_UPDATE')
	self:RegisterEvent('PLAYER_TARGET_CHANGED')
	self:RegisterEvent('PLAYER_FOCUS_CHANGED')
	self:RegisterEvent('UNIT_PET')
	self:ScheduleRepeatingEvent("UpdateCoverGauge", self.UpdateGauge, 0.05, self)
	SphereCoverBG:Show()
	SphereCover:Show()
end

--- ========================================================= ---
---  On Disable
--- ========================================================= ---
function Sphere_CoverGauge:OnDisable()
	self:UnregisterAllEvents();
	SphereCoverBG:Hide()
	SphereCover:Hide()
end

--- ========================================================= ---
---  Add Event Type
--- ========================================================= ---
function Sphere_CoverGauge:AddEventType(func, key, value)
	func[key] = value
end

--- ========================================================= ---
---  Remove Event Type
--- ========================================================= ---
function Sphere_CoverGauge:RemoveEventType(func, key)
	func[key] = nil
end

--- ========================================================= ---
--- UNIT_PORTRAIT_UPDATE
--- ========================================================= ---
function Sphere_CoverGauge:UNIT_PORTRAIT_UPDATE()
	self:SetGaugePortrait()
end

--- ========================================================= ---
--- PLAYER_TARGET_CHANGED
--- ========================================================= ---
function Sphere_CoverGauge:PLAYER_TARGET_CHANGED()
	self:SetGaugePortrait()
end

--- ========================================================= ---
--- PLAYER_FOCUS_CHANGED
--- ========================================================= ---
function Sphere_CoverGauge:PLAYER_FOCUS_CHANGED()
	self:SetGaugePortrait()
end

--- ========================================================= ---
--- UNIT_PET
--- ========================================================= ---
function Sphere_CoverGauge:UNIT_PET()
	self:SetGaugePortrait()
end

--- ========================================================= ---
--- Create Sphere Frames
--- ========================================================= ---
function Sphere_CoverGauge:CreateFrames()
	Sphere:CreateFrame("Button", "SphereCoverBG", SphereMainFrame, 3, "Interface\\Addons\\Sphere\\art\\spherecover")
	Sphere:CreateFrame("Button", "SphereCover", SphereMainFrame, 4, "Interface\\Addons\\Sphere\\art\\spherecover")
end

--- ========================================================= ---
---  Set Gauge Type
--- ========================================================= ---
function Sphere_CoverGauge:SetGaugeType(value)
	if value then
		self.db.char.gaugeType = value
		self.db.char.gaugeCustom = eventType[value]
	else
		value = self.db.char.gaugeType
	end
		
	if value == "health" or value == "power" or value == "threat" or value == "experience" or value == "reputation" or value == "custom" or value == "none" then
		 self.db.char.useCustom = false
	else
		self.db.char.useCustom = true
	end
end

--- ========================================================= ---
---  Set Gauge Unit
--- ========================================================= ---
function Sphere_CoverGauge:SetGaugeUnit(value)
	if value then
		self.db.char.gaugeUnit = value
	else
		value = self.db.char.gaugeUnit
	end
end

--- ========================================================= ---
---  Set Custom Gauge
--- ========================================================= ---
function Sphere_CoverGauge:SetCustomGauge(value)
	if value then
		if self.db.char.gaugeType == "custom" then
			self.db.char.gaugeCustom = value
		else
			self.db.char.gaugeCustom = eventType[self.db.char.gaugeType]
		end
	else
		value = self.db.char.gaugeCustom
	end
end

--- ========================================================= ---
---  Set Gauge Portrait
--- ========================================================= ---
function Sphere_CoverGauge:SetGaugePortrait()
	value = self.db.char.portrait

	if value then
		SetPortraitTexture(SphereCover:GetNormalTexture(), self.db.char.gaugeUnit);
		SetPortraitTexture(SphereCoverBG:GetNormalTexture(), self.db.char.gaugeUnit);
	else
		SetPortraitTexture(SphereCover:GetNormalTexture(), "");
		SetPortraitTexture(SphereCoverBG:GetNormalTexture(), "");
		SetPortraitToTexture(SphereCover:GetNormalTexture(), "Interface\\Addons\\Sphere\\art\\spherecover");
	end
end

--- ========================================================= ---
--- Update Gauge
--- ========================================================= ---
function Sphere_CoverGauge:UpdateGauge()
	local sPercent = 0
	local r, g, b, a
	
	if self.db.char.useCustom then
		sPercent, r, g, b, a  = Sphere[self.db.char.gaugeType](Sphere)
	else
		if self.db.char.gaugeType == "power" then
			local powerType = DogTag:Evaluate(self.db.char.gaugeUnit, "[TypePower]")
			if powerType == "" then powerType = "Mana" end
			r, g, b, a = unpack(Sphere.db.char.colors[string.lower(powerType)])
		else
			r, g, b, a = unpack(Sphere.db.char.colors[self.db.char.gaugeType])
		end
		
		if tonumber(DogTag:Evaluate(self.db.char.gaugeUnit, self.db.char.gaugeCustom)) then
			sPercent = tonumber(DogTag:Evaluate(self.db.char.gaugeUnit, self.db.char.gaugeCustom))
		else
			sPercent = 0
		end
		
		if self.db.char.animate then
			if (start == nil) then
				start = sPercent;
			end
			if (start > sPercent) then
				start = start - 1;
				if (start < sPercent) then
					start = sPercent;
				end
			else
				start = start + 1;
				if (start > sPercent) then
					start = sPercent;
				end
			end
			sPercent = start;
		end
	end
	
	if (sPercent > 100) then
		sPercent = 100;
	end
	
	if (sPercent < 0) then
		sPercent = 0;
	end
	
	if Sphere:HasModule('InnerGauge') then
		if Sphere:IsModuleActive('InnerGauge') then
			SphereCover:SetWidth(31);
			SphereCover:SetHeight(31);
			SphereCoverBG:SetWidth(31);
			SphereCoverBG:SetHeight(31);
		else
			SphereCover:SetWidth(45);
			SphereCover:SetHeight(45);
			SphereCoverBG:SetWidth(45);
			SphereCoverBG:SetHeight(45);
		end
	else 
		SphereCover:SetWidth(45);
		SphereCover:SetHeight(45);
		SphereCoverBG:SetWidth(45);
		SphereCoverBG:SetHeight(45);
	end

	SphereCover:GetNormalTexture():SetVertexColor(r, g, b, a);	
	SphereCover:SetAlpha(sPercent);
	
end