--- ========================================================= ---
---  Setup Revision, Version, Date and add to AceLibrary
--- ========================================================= ---
local VERSION = tonumber(("$Revision: 43977 $"):match("%d+"))

local Sphere = Sphere
local Sphere_SubButtons = Sphere:NewModule("SubButtons", "AceEvent-2.0", "AceHook-2.1")
local self = Sphere_SubButtons
if Sphere.revision < VERSION then
	Sphere.version = "r" .. VERSION
	Sphere.revision = VERSION
	Sphere.date = ("$Date: 2007-07-17 03:04:53 -0400 (Tue, 17 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
Sphere.desc = "Adds Buttons to the Sphere"

--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local L = Sphere:L("Sphere_SubButtons")

local data = {
	menuCount = 0,
	subCount = 0,
	cursorType = nil,
	cursorInfo1 = nil,
	cursorInfo2 = nil,
	cursorHasItem = false,
}

--- ========================================================= ---
---  On Intitalize / Setup Database
--- ========================================================= ---
function Sphere_SubButtons:OnInitialize()
	self.db = Sphere:AcquireDBNamespace("SubButtons")
	Sphere:RegisterDefaults("SubButtons", "char", {
		settings = {
			scale = 1.00,
			offSet = 5,
		},
		SphereSubBar1 = {
			buttons = {},
		},
		SphereSubBar2 = {
			buttons = {},
		},
		SphereSubBar3 = {
			buttons = {},
		},
		SphereSubBar4 = {
			buttons = {},
		},
		SphereSubBar5 = {
			buttons = {},
		},
		SphereSubBar6 = {
			buttons = {},
		},
		SphereSubBar7 = {
			buttons = {},
		},
		SphereSubBar8 = {
			buttons = {},
		},
		SphereSubBar9 = {
			buttons = {},
		},
		SphereSubBar10 = {
			buttons = {},
		},
		
	})

	Sphere_SubButtons:CreateFrames()
end

--- ========================================================= ---
---  Setup Options
--- ========================================================= ---
Sphere_SubButtons:RegisterSphereOption({
	name = L["Sub Buttons"],
	desc = L["Change the Sub Buttons options."],
	type = 'group',
	args = {
		appearance = {
			name = L["Appearance"],
			desc = L["Change how the Sub Buttons Appearance looks in general."],
			type = 'group',
			args = {
				size = {
					name = L["Size"],
					desc = L["Set how large the Sub Buttons are."],
					type = 'range',
					min = 0.50,
					max = 2,
					step = 0.01,
					bigStep = 0.05,
					get = function()
						return self.db.char.settings.scale
					end,
					set = "SetScale"
				},
				offSet = {
					name = L["offSet"],
					desc = L["Set how much of an offSet the Sub Buttons are."],
					type = 'range',
					min = 0,
					max = 20,
					step = 1,
					bigStep = 2,
					get = function()
						return self.db.char.settings.offSet
					end,
					set = "SetOffSet"
				},
			},
		},
	}

})

--- ========================================================= ---
---  On Enable
--- ========================================================= ---
function Sphere_SubButtons:OnEnable()

end

--- ========================================================= ---
---  On Disable
--- ========================================================= ---
function Sphere_SubButtons:OnDisable()
	self:UnregisterAllEvents();
end

--- ========================================================= ---
---  Set Scale
--- ========================================================= ---
function Sphere_SubButtons:SetScale(value)
	if value then
		self.db.char.settings.scale = value
	else
		value = self.db.char.settings.scale
	end
	
	for i=1, 10 do
		getglobal("SphereSubBar"..i):SetScale(value)
	end
end

--- ========================================================= ---
---  Set offSet
--- ========================================================= ---
function Sphere_SubButtons:SetOffSet(value)
	if value then
		self.db.char.settings.offSet = value
	else
		value = self.db.char.settings.offSet
	end
	
	local offsetY = {38, 8, 0, 0, -8, -38, -8, 0, 0, 8}
	for i=1, 10 do
		local xDir = 1;
		if (i > 6) then
			xDir = -1;
		end
	
		getglobal("SphereSubBar"..i):ClearAllPoints()
		if (xDir == 1) then
			getglobal("SphereSubBar"..i):SetPoint("LEFT", "SphereBar"..i, "RIGHT", value, offsetY[i]);
		else
			getglobal("SphereSubBar"..i):SetPoint("RIGHT", "SphereBar"..i, "LEFT", -value, offsetY[i]);
		end
	end
	
	for iM = 1, 10 do
		local xDir = 1;
		if (iM > 6) then
			xDir = -1;
		end
		
		for i = 1, 10 do
			getglobal("SphereSubBar"..iM.."Button"..i):ClearAllPoints()
			if (xDir == 1) then
				if (i ~= 1) then
					getglobal("SphereSubBar"..iM.."Button"..i):SetPoint("LEFT", "SphereSubBar"..iM.."Button"..(i - 1), "RIGHT", value, 0);
				else
					getglobal("SphereSubBar"..iM.."Button"..i):SetAllPoints("SphereSubBar"..iM);
				end
			else
				if (i ~= 1) then
					getglobal("SphereSubBar"..iM.."Button"..i):SetPoint("RIGHT", "SphereSubBar"..iM.."Button"..(i - 1), "LEFT", -value, 0);
				else
					getglobal("SphereSubBar"..iM.."Button"..i):SetAllPoints("SphereSubBar"..iM);
				end
			end
		end
	end
end

--- ========================================================= ---
---  Create Frames
--- ========================================================= ---
function Sphere_SubButtons:CreateFrames()
	for iM = 1, 10 do
		local bar = Sphere_Buttons:CreateBar(iM, "SphereSubBar"..iM, "SphereMainFrame", "SphereBar"..iM, "Buttons")
		for i = 1, 10 do
			local btn = Sphere_Buttons:CreateButton(i, "SphereSubBar"..iM, "SphereMainFrame", "SphereSubBar"..iM, "SphereSubBar"..iM, "SubButtons")

			btn:Show();
			
			bar:SetAttribute("addchild", btn);
		end
		bar:SetScale(self.db.char.settings.scale)
		bar = Sphere_Buttons:UpdateBar(bar)
		
		bar:SetAttribute("showstates", "!0,1");
		getglobal("SphereBar"..iM.."Header"):SetAttribute("addchild", bar);
		getglobal("SphereBar"..iM.."Button1"):SetAttribute("addchild", bar);
	end
	Sphere_SubButtons:SetOffSet()
end